﻿using ProjectMotorboat.Drownings;
using ProjectMotorboat.MovementStrategy;

namespace ProjectMotorboat;

public partial class FormMotorboat : Form
{
    private DrawningBoat? _drawningBoat;
    private AbstractStrategy? _strategy;
    public FormMotorboat()
    {
        InitializeComponent();
        _strategy = null;
    }
    private void Draw()
    {
        if (_drawningBoat == null)
        {
            return;
        }
        Bitmap bmp = new(pictureBoxMotorboat.Width, pictureBoxMotorboat.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningBoat.DrawTransport(gr);
        pictureBoxMotorboat.Image = bmp;
    }
    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningBoat):
                _drawningBoat = new DrawningBoat(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningMotorboat):
                _drawningBoat = new DrawningMotorboat(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningBoat.SetPictureSize(pictureBoxMotorboat.Width, pictureBoxMotorboat.Height);
        _drawningBoat.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }
    private void ButtonCreate_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningMotorboat));


    private void ButtonBoat_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningBoat));

    


    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningBoat == null)
        {
            return;
        }
        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningBoat.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningBoat.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningBoat.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningBoat.MoveTransport(DirectionType.Right);
                break;
        }
        if (result)
        {
            Draw();
        }
    }

    private void ButtonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawningBoat == null)
            {
                return;
            }

            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new MoveableBoat(_drawningBoat), pictureBoxMotorboat.Width, pictureBoxMotorboat.Height);
            }

            if (_strategy == null)
            {
                return;
            }

            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
    }

